from abc import ABCMeta


class Source(object):
    """
    Orchestrates the application of a deserialization format on the input source
    """
    def __init__(self, source_impl, deserializer, license=None):
        self.source_impl = source_impl
        self.deserializer = deserializer
        self.license = license

    def pull(self):
        with self.source_impl as source:
            raw = source.pull()
            for item in self.deserializer(raw):
                yield item


class BaseSource(object):
    """
    Base class for all input source implementations. This base also implements a context manager
    interface to allow implementations to initialize and clean up in a safe manner, e.g. removing
    scratch files or closing network connections.
    """
    __metaclass__ = ABCMeta

    def __enter__(self):
        """
        Initialization function before data is pulled. Note that this should always return `self`
        """
        return self

    def pull(self):
        """
        Pulls the data into a input stream. This should return a file-like object
        """
        raise NotImplementedError

    def __exit__(self, *args):
        """
        Clean-up function that is called after data is pulled, or when an error occurs while pulling
        the data
        """
        pass



from landgrab.source import BaseSource


class FileSource(BaseSource):
    """
    An input source for local files
    """
    def __init__(self, uri):
        self.fn = uri[len('file://'):]

    def __enter__(self):
        self.f = open(self.fn)
        return self

    def pull(self):
        return self.f

    def __exit__(self, *args):
        return self.f.close()



import ijson
import os
import shapefile
import shutil
import tempfile
import zipfile

from landgrab.format import BaseFormat


def _find_shp(d):
    for dirpath, _, files in os.walk(d):
        for f in files:
            fpath = os.path.join(dirpath, f)
            if f.endswith('.shp'):
                return fpath
    raise IOError('No .shp file found in directory %s' % d)


class GeoJSONFormat(BaseFormat):
    def deserialize(self, raw):
        """
        Deserializes from a raw input stream a new stream of GeoJSON Features
        """
        features = ijson.items(raw, 'features.item')
        for feature in features:
            yield feature

    def serialize(self, item):
        raise NotImplementedError


class ShapefileFormat(BaseFormat):
    def deserialize(self, raw):
        """
        Deserializes a raw stream of data from a ZIP archive containing shape files. The ZIP must at
        least include a .shp, .dbf, and .shx file to work using this format.

        The resulting output stream emits GeoJSON-style Feature dictionaries.
        """
        outfolder = tempfile.mkdtemp()
        try:
            with zipfile.ZipFile(raw) as zf:
                zf.extractall(outfolder)
            shpfn = _find_shp(outfolder)
            shp = shapefile.Reader(shpfn)
            fields = [field[0] for field in shp.fields[1:]]
            for shape_record in shp.iterShapeRecords():
                yield {
                    'geometry': shape_record.shape.__geo_interface__,
                    'properties': dict(zip(fields, shape_record.record))
                }
        finally:
            shutil.rmtree(outfolder)

    def serialize(self, item):
        raise NotImplementedError



from landgrab.config import create_sink, create_source, create_task, get_deserializer, \
                            get_serializer, DEFAULT_FORMAT
from landgrab.sink import Sink
from landgrab.source import Source
from landgrab.transform import Transform


def _create_sink(sink_cfg):
    sink_impl = create_sink(sink_cfg['uri'], sink_cfg.get('params'))
    serializer = get_serializer(sink_cfg.get('format', DEFAULT_FORMAT))
    return Sink(sink_impl, serializer)


def _create_source(source_cfg):
    source_impl = create_source(source_cfg['uri'], source_cfg.get('params'))
    deserializer = get_deserializer(source_cfg.get('format', DEFAULT_FORMAT))
    return Source(source_impl, deserializer)


def _create_transform(transform_cfg):
    tasks = map(create_task, transform_cfg.get('tasks', []))
    return Transform(tasks)


class Engine(object):
    """
    The engine that orchestrates the end-to-end LandGrab job
    """
    def __init__(self, cfg):
        self.cfg = cfg

    def plan(self):
        """
        Builds out the plan for how to execute the job from the configuration contents
        """
        source = _create_source(self.cfg['source'])
        transform = _create_transform(self.cfg.get('transform', {}))
        sink = _create_sink(self.cfg['sink'])
        return source, transform, sink

    def run(self):
        """
        Actually runs the job in the following steps:
          1. Pulls the input source
          2. Applies any transformation tasks
          3. Saves the transformed data to the sink
        """
        source, transform, sink = self.plan()
        source_data = source.pull()
        transformed_data = transform.apply(source_data)
        sink.save(transformed_data)


def run(cfg):
    engine = Engine(cfg)
    engine.run()



from landgrab.format import BaseFormat


class RawFormat(BaseFormat):
    def deserialize(self, raw):
        """
        Yields a new stream of the input data, untouched
        """
        for chunk in raw:
            yield chunk

    def serialize(self, item):
        """
        Identity function
        """
        return item



from landgrab.config import FORMATS, SINKS, SOURCES, TASKS


def load():
    FORMATS['geojson'] = 'landgrab.contrib.geo.formats.GeoJSONFormat'
    FORMATS['shp'] = 'landgrab.contrib.geo.formats.ShapefileFormat'
    FORMATS['xls'] = 'landgrab.contrib.xls.formats.XLSFormat'
    SINKS['elastic'] = 'landgrab.contrib.elastic.sinks.ElasticSink'
    SOURCES['sql'] = 'landgrab.contrib.db.sources.SQLSource'
    TASKS['buffer_geometry'] = 'landgrab.contrib.geo.tasks.BufferGeometryTask'
    TASKS['project_geometry'] = 'landgrab.contrib.geo.tasks.ProjectGeometryTask'



# -*- coding: utf-8 -*-
import os.path
import click

import landgrab
import landgrab.config as config
import landgrab.contrib as contrib
import landgrab.engine as engine
import landgrab.env as env


CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
CWD = os.path.abspath(os.getcwd())


@click.command('landgrab', short_help='Geospatial data hoarding system',
               context_settings=CONTEXT_SETTINGS)
@click.version_option(version=landgrab.__version__, prog_name='landgrab')
@click.option('--env-file', '-e', type=click.Path(exists=False, dir_okay=False,
                                                  readable=True, resolve_path=True),
              default=os.path.expanduser('~/.landgrab/config.yaml'))
@click.option('--config-file', '-c', type=click.Path(exists=True, dir_okay=False, readable=True,
                                                     resolve_path=True), required=True)
@click.option('--debug', '-d', is_flag=True)
def run(env_file, config_file, debug):
    """Geospatial data hoarding system
    """
    env.load(env_file)
    contrib.load()
    cfg = config.load(config_file)
    engine.run(cfg)


def main():
    run()


if __name__ == '__main__':
    main()



from abc import ABCMeta


class BaseTask(object):
    """
    Base interface for all transformation tasks
    """
    __metaclass__ = ABCMeta

    def __call__(self, item):
        """
        Function that should return the transformed item
        """
        raise NotImplementedError



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from landgrab.source import BaseSource


def _compose_sql_uri(uri, dialect, driver):
    return uri.replace('sql://', '%s+%s://' % (dialect, driver))


def _start_session(sql_uri):
    """
    Starts a session with the database
    """
    engine = create_engine(sql_uri)
    session_factory = sessionmaker()
    session_factory.configure(bind=engine)
    session = session_factory()
    return session


class SQLSource(BaseSource):
    """
    An input source for data stored in a SQL database
    """
    def __init__(self, uri, query, dialect='postgresql', driver='psycopg2'):
        self.uri = uri
        self.query = query
        self.dialect = dialect
        self.driver = driver

    def __enter__(self):
        sql_uri = _compose_sql_uri(self.uri, self.dialect, self.driver)
        self.session = _start_session(sql_uri)
        return self

    def pull(self):
        return self.session.execute(self.query)

    def __exit__(self, *args):
        self.session.close()



from importlib import import_module
import os
import yaml


def import_class(class_path):
    mod, clazz = class_path.rsplit('.', 1)
    return getattr(import_module(mod), clazz)


DEFAULT_FORMAT = 'raw'
# TODO: Integrate these mappings into a more pluggable interface
FORMATS = {
    'jsonlines': 'landgrab.format.jsonlines.JSONLinesFormat',
    'raw': 'landgrab.format.raw.RawFormat',
    'rows': 'landgrab.format.rows.RowsFormat',
}
SOURCES = {
    'http': 'landgrab.source.http.HTTPSource',
    'https': 'landgrab.source.http.HTTPSource',
    'file': 'landgrab.source.file.FileSource',
    's3': 'landgrab.source.s3.S3Source',
}
TASKS = {
    'extract': 'landgrab.task.dict.ExtractTask',
    'rename_key': 'landgrab.task.dict.RenameKeyTask',
    'project': 'landgrab.task.dict.ProjectTask',
    'filter': 'landgrab.task.dict.FilterTask',
}
SINKS = {
    'file': 'landgrab.sink.file.FileSink',
    's3': 'landgrab.sink.s3.S3Sink',
}


class ConfigError(Exception):
    """
    Occurs when the job configuration is malformed or invalid
    """
    pass


def _render(config_file):
    """
    Expands shell-style environment variables found in the configuration file, e.g. `$USER`
    """
    with open(config_file) as f:
        return os.path.expandvars(f.read())


# TODO: Implement configuration class
def load(config_file):
    """
    Loads a configuration file into memory
    """
    config = _render(config_file)
    return yaml.load(config)


def get_deserializer(format):
    """
    Gets the deserialization function specified by the `format`
    """
    if format not in FORMATS:
        raise ConfigError('No such format: \'%s\'' % format)
    ctor = import_class(FORMATS[format])
    return ctor().deserialize


def get_serializer(format):
    """
    Gets the serialization function specified by the `format`
    """
    if format not in FORMATS:
        raise ConfigError('No such format: \'%s\'' % format)
    ctor = import_class(FORMATS[format])
    return ctor().serialize


def create_source(uri, params=None):
    """
    Constructs an input source from the URI spec and any additional source parameters. Note that
    when the URI does not have a scheme, the "file" scheme is assumed.
    """
    if ':' not in uri:
        uri = 'file://%s' % uri
    proto = uri.split(':')[0]
    if proto not in SOURCES:
        raise ConfigError('No such source: \'%s\'' % proto)
    ctor = import_class(SOURCES[proto])
    if params is None:
        params = {}
    return ctor(uri, **params)


def create_sink(uri, params):
    """
    Constructs an output sink from the URI spec and any additional sink parameters. Note that when
    the URI does not have a scheme, the "file" scheme is assumed.
    """
    if ':' not in uri:
        uri = 'file://%s' % uri
    proto = uri.split(':')[0]
    if proto not in SINKS:
        raise ConfigError('No such sink: \'%s\'' % proto)
    ctor = import_class(SINKS[proto])
    if params is None:
        params = {}
    return ctor(uri, **params)


def create_task(task_def):
    """
    Constructs a transformation task from the provided task definition
    """
    task_type = task_def['type']
    if task_type not in TASKS:
        raise ConfigError('No such task: \'%s\'' % task_type)
    ctor = import_class(TASKS[task_type])
    kwargs = task_def.copy()
    del kwargs['type']
    return ctor(**kwargs)



from elasticsearch import helpers, Elasticsearch
from objectpath import Tree

from landgrab.sink import BaseSink


def _generate_docs(docs, index, doc_type, doc_id_field):
    for doc in docs:
        t = Tree(doc)
        yield {
            '_index': index,
            '_type': doc_type,
            '_id': t.execute(doc_id_field),
            '_source': doc
        }


class ElasticSink(BaseSink):
    def __init__(self, uri, index, doc_type, doc_id_field, chunk_size=500):
        self.uri = uri[len('elastic://'):]
        self.index = index
        self.doc_type = doc_type
        self.doc_id_field = doc_id_field
        self.chunk_size = chunk_size

    def __enter__(self):
        self.es = Elasticsearch(hosts=self.uri, timeout=300)
        return self

    def save_stream(self, items):
        docs = _generate_docs(items, self.index, self.doc_type, self.doc_id_field)
        helpers.bulk(self.es, docs, chunk_size=self.chunk_size, refresh=True)

    def __exit__(self, *args):
        pass



import simplejson as json

from landgrab.format import BaseFormat


class JSONLinesFormat(BaseFormat):
    def deserialize(self, raw):
        """
        Deserializes from a raw input stream a new stream of JSON dictionaries
        """
        for line in raw:
            yield json.loads(line)

    def serialize(self, item):
        """
        Serializes each item as a JSON-encoded string with a line ending
        """
        return json.dumps(item) + '\n'



from abc import ABCMeta


class BaseFormat(object):
    """
    Base class for all input/output formats
    """
    __metaclass__ = ABCMeta

    def serialize(self, item):
        pass

    def deserialize(self, raw):
        pass



from abc import ABCMeta


class Sink(object):
    def __init__(self, sink_impl, serializer):
        self.sink_impl = sink_impl
        self.serializer = serializer

    def serialized_data(self, transformed_data):
        for item in transformed_data:
            yield self.serializer(item)

    def save(self, transformed_data):
        with self.sink_impl as sink:
            sink.save_stream(self.serialized_data(transformed_data))


class BaseSink(object):
    __metaclass__ = ABCMeta

    def __enter__(self):
        return self

    def save_stream(self, items):
        for item in items:
            self.save(item)

    def save(self, item):
        raise NotImplementedError

    def __exit__(self, *args):
        pass



from landgrab.format import BaseFormat


class RowsFormat(BaseFormat):
    def deserialize(self, raw):
        """
        Deserializes from a raw input stream a new stream of dictionaries
        """
        for line in raw:
            yield dict(line)

    def serialize(self, item):
        raise NotImplementedError



import smart_open
import urlparse
import urllib


from landgrab.sink import BaseSink


def _parse_uri(uri):
    """
    Parses an S3 URI
    """
    parts = urlparse.urlsplit(uri)
    netloc = parts.hostname
    path = urllib.unquote_plus(parts.path[1:])
    username = urllib.unquote_plus(parts.username) if parts.username else None
    password = urllib.unquote_plus(parts.password) if parts.password else None
    return username, password, netloc, path


def _compose_uri(access_key_id, secret_access_key, bucket, key):
    return 's3://%s:%s@%s/%s' % (access_key_id, secret_access_key, bucket, key)


class S3Sink(BaseSink):
    def __init__(self, uri, access_key_id=None, secret_access_key=None):
        username, password, bucket, key = _parse_uri(uri)
        self.access_key_id = access_key_id if access_key_id else username
        self.secret_access_key = secret_access_key if secret_access_key else password
        self.bucket = bucket
        self.key = key

    def __enter__(self):
        s3_uri = _compose_uri(self.access_key_id, self.secret_access_key, self.bucket, self.key)
        self.upload_stream = smart_open.smart_open(s3_uri, mode='wb')
        return self

    def save(self, item):
        self.upload_stream.write(item)

    def __exit__(self, *args):
        self.upload_stream.close()



import codecs
import json
import requests
import tempfile
import logging

from landgrab.source import BaseSource

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)


class HTTPSource(BaseSource):
    """
    An input source for HTTP-based network data
    """
    def __init__(self, uri, method='GET', query_params=None, results_key=None, username=None,
                 password=None, pagination=None):
        self.uri = uri
        self.method = method
        if pagination:
            self.paginate = True
            self.results_per_page = pagination.get('results_per_page', 25)
            self.results_per_page_param = pagination.get('results_per_page_param', None)
            self.page_offset_param = pagination.get('page_offset_param', None)
            self.max_pages = pagination.get('max_pages', 10)
            self.current_page = pagination.get('current_page', 0)
        else:
            self.paginate = False
        if query_params:
            self.query_params = query_params
        else:
            self.query_params = {}
        self.results_key = results_key
        self.username = username
        self.password = password

    def _make_request(self, query_params=None):
        if not query_params:
            query_params = self.query_params
        if self.username:
            r = requests.get(
                self.uri,
                auth=(self.username, self.password),
                params=query_params
            )
        else:
            r = requests.get(
                self.uri,
                params=query_params
            )
        LOGGER.debug('Making HTTP request to: %s', r.url)
        return r

    def _pagination_query_param_generator(self):
        page_offset = self.current_page
        for i in range(self.max_pages):
            query_params = {
                self.results_per_page_param: self.results_per_page,
                self.page_offset_param: page_offset
            }
            query_params.update(self.query_params)
            yield query_params
            page_offset += self.results_per_page

    def __enter__(self):
        self.f = tempfile.NamedTemporaryFile(delete=True)
        with codecs.open(self.f.name, mode='w', encoding='utf-8') as tmpf:
            if self.paginate:
                for query_params in self._pagination_query_param_generator():
                    r = self._make_request(query_params)
                    if r.status_code < 400:
                        response = r.json()
                        if self.results_key:
                            results = response[self.results_key]
                        else:
                            results = response
                        if results:
                            for result in results:
                                tmpf.write(json.dumps(result))
                                tmpf.write('\n')
                        else:
                            LOGGER.debug('No results for: %s', r.url)
                            break
                    else:
                        LOGGER.debug('Error[%s] on HTTP request to: %s', (r.status_code, r.url))
                        break
            else:
                r = self._make_request()
                if r.status_code < 400:
                    response = r.json()
                    tmpf.write(json.dumps(response))
                else:
                    LOGGER.debug('Error[%s] on HTTP request to: %s', (r.status_code, r.url))
            return self

    def pull(self):
        return self.f

    def __exit__(self, *args):
        self.f.close()



from landgrab.sink import BaseSink


class FileSink(BaseSink):
    def __init__(self, uri):
        self.fn = uri[len('file://'):]

    def __enter__(self):
        self.f = open(self.fn, 'w')
        return self

    def save(self, item):
        self.f.write(item)

    def __exit__(self, *args):
        self.f.close()



import os
import sys
import yaml

from landgrab.config import FORMATS, SOURCES, SINKS, TASKS


DEFAULT_PLUGIN_PATH = '~/.landgrab/plugins/'


def load(config_file):
    if not os.path.exists(config_file):
        return

    with open(config_file) as f:
        config = yaml.load(f)

    # Inject plugins to be used by job configurations
    plugins = config.get('plugins')
    sys.path.append(os.path.expanduser(DEFAULT_PLUGIN_PATH))
    for extra_path in plugins.get('python_path', []):
        sys.path.append(extra_path)
    if plugins.get('formats'):
        FORMATS.update(plugins['formats'])
    if plugins.get('sources'):
        SOURCES.update(plugins['sources'])
    if plugins.get('sinks'):
        SINKS.update(plugins['sinks'])
    if plugins.get('tasks'):
        TASKS.update(plugins['tasks'])



from functools import partial
import pyproj
from shapely.geometry import shape, mapping
from shapely.ops import transform

from landgrab.task import BaseTask


class BufferGeometryTask(BaseTask):
    """
    Buffers the geometry of a GeoJSON feature by a configurable buffer size amount, e.g.:

    - type: buffer_geometry
      buffer_size: 1

    This is useful when complex geometries have hard-to-detect inconsistencies in their coordinate
    rings, e.g. overlapping, closeness, etc.
    """
    def __init__(self, buffer_size=0):
        self.buffer_size = buffer_size

    def __call__(self, item):
        geom = item['geometry']
        s = shape(geom).buffer(self.buffer_size)
        buffered_geom = mapping(s)
        item['geometry'] = buffered_geom
        return item


class ProjectGeometryTask(BaseTask):
    """
    Projects the geometry of a GeoJSON feature to a configurable spatial reference system, e.g.:

    - type: project_geometry
      in_system: EPSG:26986
      out_system: EPSG:4326
    """
    def __init__(self, in_system, out_system):
        self.in_system = in_system
        self.out_system = out_system

    def __call__(self, item):
        geom = item['geometry']
        project = partial(
            pyproj.transform,
            pyproj.Proj(init=self.in_system),
            pyproj.Proj(init=self.out_system, preserve_units=True)
        )
        s = transform(
            project,
            shape(geom)
        )
        projected_geom = mapping(s)
        item['geometry'] = projected_geom
        return item



import shutil
import smart_open
import tempfile
import urlparse
import urllib


from landgrab.source import BaseSource


def _parse_uri(uri):
    """
    Parses an S3 URI
    """
    parts = urlparse.urlsplit(uri)
    netloc = parts.hostname
    path = urllib.unquote_plus(parts.path[1:])
    username = urllib.unquote_plus(parts.username) if parts.username else None
    password = urllib.unquote_plus(parts.password) if parts.password else None
    return username, password, netloc, path


def _compose_uri(access_key_id, secret_access_key, bucket, key):
    """
    Creates an S3 URI from the required fields; the inverse operation of `_parse_uri`
    """
    return 's3://%s:%s@%s/%s' % (access_key_id, secret_access_key, bucket, key)


class S3Source(BaseSource):
    """
    An input source for data blobs stored in S3
    """
    # TODO: Support glob URIs?
    def __init__(self, uri, access_key_id=None, secret_access_key=None):
        username, password, bucket, key = _parse_uri(uri)
        self.access_key_id = access_key_id if access_key_id else username
        self.secret_access_key = secret_access_key if secret_access_key else password
        self.bucket = bucket
        self.key = key

    def __enter__(self):
        s3_uri = _compose_uri(self.access_key_id, self.secret_access_key, self.bucket, self.key)
        self.f = tempfile.NamedTemporaryFile(delete=True)
        self.download_stream = smart_open.smart_open(s3_uri, mode='rb')
        return self

    def pull(self):
        shutil.copyfileobj(self.download_stream, self.f)
        self.f.flush()
        self.f.seek(0)
        return self.f

    def __exit__(self, *args):
        self.f.close()



import simplejson
import pandas as pd

from landgrab.format import BaseFormat


class XLSFormat(BaseFormat):
    def deserialize(self, raw):
        """
        Deserializes from a raw input stream a new stream of XLS rows
        """
        df = pd.read_excel(raw, skiprows=1)
        for row in simplejson.loads(df.to_json(orient='records')):
            yield row

    def serialize(self, item):
        raise NotImplementedError



from objectpath import Tree

from landgrab.task import BaseTask


def _maybe_insert_nested(d, k, v):
    """
    Tries to insert a value into a dictionary at a nested key using dot notation, e.g. "a.b" would
    refer to the key "b" inside of the dictionary at key "a" inside of a dictionary
    """
    if '.' in k:
        sub_dict_k, k = k.split('.', 1)
        if sub_dict_k not in d:
            d[sub_dict_k] = {}
        sub_d = d[sub_dict_k]
        _maybe_insert_nested(sub_d, k, v)
    else:
        d[k] = v


class ProjectTask(BaseTask):
    """
    Projects an input dictionary into a new dictionary; similar to a SQL-like SELECT statement. The
    schema should be specified as a dictionary where the key is the output key and the value is an
    ObjectPath expression that refers to the input key, e.g.:

    - type: project
      schema:
        newkey: $.properties.oldkey

    """
    def __init__(self, schema):
        self.schema = schema

    def __call__(self, item):
        output = {}
        t = Tree(item)
        for k, expression in self.schema.items():
            _maybe_insert_nested(output, k, t.execute(expression))
        return output


class ExtractTask(BaseTask):
    """
    Extracts a single value from the input dictionary, as specified by the provided ObjectPath
    expression, e.g.:

    - type: extract
      expression: $.properties.geometry

    Note that the resulting value becomes the output item that is processed by any subsequent tasks
    """
    def __init__(self, expression):
        self.expression = expression

    def __call__(self, item):
        t = Tree(item)
        return t.execute(self.expression)


class RenameKeyTask(BaseTask):
    """
    Renames a set of keys to a set of new keys, as specified by the `map` property, e.g.:

    - type: rename
      map:
        oldkey: newkey
        oldkey2: newkey2
    """
    def __init__(self, map):
        self.map = map

    def __call__(self, item):
        for old_key, new_key in self.map.items():
            item[new_key] = item[old_key]
            del item[old_key]
        return item


class FilterTask(BaseTask):
    """
    Filters the input dictionaries based on the value (or values) of a specific key as specified by
    the provided ObjectPath expression; similar to a SQL-like WHERE statement, e.g.:

    - type: filter
      expression: $.properties.fclass
      value: building
    """

    def __init__(self, expression, value):
        self.expression = expression
        self.value = value

    def __call__(self, item):
        t = Tree(item)
        v = t.execute(self.expression)
        if type(self.value) == list:
            if v in self.value:
                return item
            else:
                return None
        else:
            if v == self.value:
                return item
            else:
                return None



#!/usr/bin/env python
from setuptools import setup, find_packages
from pip.req import parse_requirements
from pip.download import PipSession


SESSION = PipSession()


def requirements(fn):
    return [str(r.req) for r in parse_requirements(fn, session=SESSION)]

INSTALL_REQUIRES = requirements('./requirements.txt')
TESTS_REQUIRE = requirements('./test_requirements.txt')


setup(
    name='landgrab',
    version='0.1.9',
    description='Geospatial data hoarding system',
    author='Station A',
    author_email='software@stationa.com',
    url='https://github.com/StationA/landgrab',
    packages=find_packages(exclude=['*tests*']),
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'landgrab=landgrab.__main__:main'
        ],
    },
    install_requires=INSTALL_REQUIRES,
    extras_require={
        'db': requirements('./requirements_db.txt'),
        'elastic': requirements('./requirements_elastic.txt'),
        'xls': requirements('./requirements_xls.txt'),
        'geo': requirements('./requirements_geo.txt')
    },
    tests_require=TESTS_REQUIRE,
    license='License :: OSI Approved :: MIT License',
)



from abc import ABCMeta


class Transform(object):
    """
    Orchestrates the application of zero or more transformation tasks on the source data
    """
    def __init__(self, tasks):
        self.tasks = tasks

    def apply(self, source_data):
        """
        Applies each transformation task to the source data, using the result of the task as the
        input to the next task
        """
        for item in source_data:
            for task in self.tasks:
                item = task(item)
                if item is None:
                    break
            if item is not None:
                yield item
